@@ -13,9 +13,16 @@ use Lemonldap::NG::Common::Regexp;
 #inherits Lemonldap::NG::Portal::OpenID::Server
 #link Lemonldap::NG::Portal::OpenID::SREG protected sreg_extension
 
-our $VERSION = '0.992';
+our $VERSION = '1.0.0';
 our $initDone;
 
+BEGIN {
+    eval {
+        require threads::shared;
+        threads::shared::share($initDone);
+    };
+}
+
 ## @method void issuerDBInit()
 # Do nothing
 # @return Lemonldap::NG::Portal error code
@@ -231,8 +238,7 @@ sub _openIDResponse {
             $self->info(
                 '<h3>'
                   . sprintf(
-                    &Lemonldap::NG::Portal::_i18n::msg(PM_OPENID_EXCHANGE),
-                    $data->{trust_root}
+                    $self->msg(PM_OPENID_EXCHANGE), $data->{trust_root}
                   )
                   . "</h3>"
             );
