@@ -12,7 +12,7 @@ use base qw(Lemonldap::NG::Portal::_DBI );
 
 #inherits Lemonldap::NG::Portal::_SMTP
 
-our $VERSION = '1.1.0';
+our $VERSION = '1.2.0';
 
 ## @apmethod int passwordDBInit()
 # Load SMTP functions and call DBI authInit()
@@ -72,11 +72,6 @@ sub modifyPassword {
 
     $self->lmLog( "Password changed for $user", 'debug' );
 
-    # Update password in session if needed
-    my $infos;
-    $infos->{_password} = $self->{newpassword};
-    $self->updateSession($infos) if ( $self->{storePassword} );
-
     PE_PASSWORD_OK;
 }
 
